#include "StdAfx.h"
#include "WASGetUserProfile.h"
#include "ConvertUTF.h"
#include "MessengerActor.h"
#include "UtilFunction.h"
#include "GameTalkAPIGlobal.h"

CWASGetUserProfile::CWASGetUserProfile(CWASActor *pActor) : 
	CWASCommand(pActor)
{
}


CWASGetUserProfile::~CWASGetUserProfile(void)
{
}


void CWASGetUserProfile::ExecuteAsync()
{
	USES_UTF8_CONVERSION;

	CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
	if(pMessengerActor == NULL)
	{
		return;
	}	

	WSTRING strURL = pMessengerActor->GetURL("SVR_OPENTALK_PROFILE", "OBJ_OPENTALK_PROFILE");

	m_httpFacade.session().SetURL( W2U8(strURL.c_str()), pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().SetMethod(HTTP_METHOD_POST);
	m_httpFacade.session().AddBodyVariable("userid", W2U8(pMessengerActor->GetUserID()));
	m_httpFacade.session().AddBodyVariable("targetid", W2U8(m_strUserID.c_str()));
	m_httpFacade.session().AddBodyVariable("ticket", pMessengerActor->GetMessengerTicket());
	m_httpFacade.session().AddBodyVariable("tickettype", pMessengerActor->GetMessengerTicketType());

	m_httpFacade.session().SetKeepRawHeader(TRUE);
	m_httpFacade.SendRequest(this);
}

bool CWASGetUserProfile::ParseAndEventFire(nfc::CIkXMLDom * pDom)
{
	if(pDom == NULL)
		return false;

	USES_UTF8_CONVERSION;

    CMessengerActor *pMessengerActor = static_cast<CMessengerActor *>(m_pActor->GetMessengerActor());
    ASSERT_IF_FAIL_RETURN_VALUE(pMessengerActor, false);
    
    CProfileImageActor *pProfileImageActor = static_cast<CProfileImageActor *>(pMessengerActor->GetProfileImageActor());
    ASSERT_IF_FAIL_RETURN_VALUE(pProfileImageActor, false);
    
	nfc::IIkXMLNode* pNode = pDom->GetdocumentElement()->selectSingleNode("node");
	if (pNode == NULL)
	{
		return false;
	}

	//  !
	bool bOpenTalkInfo = true;

	nfc::IIkXMLAttribute* pAttr;

	// 
	if ( NULL != (pAttr = pNode->getAttribute("nickname")))
		m_sUserInfo.strTalkName	= U82W(pAttr->Gettext().c_str());		

	// ڱҰ
	if ( NULL != (pAttr = pNode->getAttribute("intro")))
		m_sUserInfo.strTalkIntro = U82W(pAttr->Gettext().c_str());		

	//  ʴ  
	pAttr = pNode->getAttribute("allow_invite");
	if (pAttr)
		m_sUserInfo.eInviteAllowType = (GTInviteAllowType)atoi(pAttr->Gettext().c_str());

	//    !
	pAttr = pNode->getAttribute("open_profile");
	if (pAttr)
		bOpenTalkInfo = strcmp(pAttr->Gettext().c_str(), "Y") == 0;

	//  ̹ ̸
	if ( NULL != (pAttr = pNode->getAttribute("profimg_name")))
		m_sUserInfo.strProfileImageName = U82W(pAttr->Gettext().c_str());		
	
	//  ̹ ̸
	if ( NULL != (pAttr = pNode->getAttribute("profimg_crc")))
	{
//		DLOG("IMG CRC : %s\n", pAttr->Gettext().c_str());
		
		//DWORD temp;		
		//ImageUtil::GetImageCRCFromString(pAttr->Gettext().c_str(), temp);
  //      
		//m_sUserInfo.dwProfileImageCRC = temp;
  //      
  //      pProfileImageActor->UpdateProfileImageCRC(m_sUserInfo.strUserID.c_str(), temp);
	}
	
	return true;
}


// override in drived class
void CWASGetUserProfile::OnWasResponse(trid_t trid,
								  bool bSuccess,
								  int /*nWinINetErrorCode*/, 
								  int /*nHttpResponseCode*/, 
                                  LPCSTR /*szWasResCodeInHeader*/, 
                                  LPCSTR szWasResCodeInXml,
                                  nfc::CIkXMLDom * pDom)
{
	if(bSuccess)
	{
		if(ParseAndEventFire(pDom) == false)
			m_eError = wasError_Protocol;
	}
	
	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	GT_USER_PROFILE_DATA sData;
	m_sUserInfo.Convert(sData);

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetUserProfile(m_pActor, GetID(), m_eError, sData) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}


void CWASGetUserProfile::OnRequestTimeout(CHttpFacade * /*pHttpF*/, DWORD /*dwSeconds*/)
{
	m_eError = wasError_Timeout;

	GT_ASSERT(m_pActor);

	//CRITICAL_SECTION * pCS = m_pActor->GetCriticalSection();
	//EnterCriticalSection(pCS);

	CGTObserved<IGTWASActorEvent>::listobserver_t observers;
	m_pActor->GetObservers(&observers);
	CGTObserved<IGTWASActorEvent>::listobserver_t::iterator it = observers.begin();

	GT_USER_PROFILE_DATA sData;
	m_sUserInfo.Convert(sData);

	for (; it != observers.end(); ++it)
	{
		IGTWASActorEvent *pObserver = static_cast<IGTWASActorEvent *>(*it);
		if(pObserver->OnGTWASActor_ResponseGetUserProfile(m_pActor, GetID(), m_eError, sData) == false)
			break;
	}

	//LeaveCriticalSection(pCS);

	if(m_pActor)
		m_pActor->ReleaseCommand(GetID());
}
